@@ -1,12 +1,14 @@
 import base64
 import requests
 import time
+import yaml
 
 FTS_CONFIG = {
     # table: columns
     "commits": ["message"],
     "issue_comments": ["body"],
     "issues": ["title", "body"],
+    "pull_requests": ["title", "body"],
     "labels": ["name", "description"],
     "licenses": ["name"],
     "milestones": ["title", "description"],
@@ -148,6 +150,77 @@ def save_issues(db, issues, repo):
             table.m2m("labels", label, pk="id")
 
 
+def save_pull_requests(db, pull_requests, repo):
+    if "milestones" not in db.table_names():
+        if "users" not in db.table_names():
+            # So we can define the foreign key from milestones:
+            db["users"].create({"id": int}, pk="id")
+        db["milestones"].create(
+            {"id": int, "title": str, "description": str, "creator": int, "repo": int},
+            pk="id",
+            foreign_keys=(("repo", "repos", "id"), ("creator", "users", "id")),
+        )
+    for original in pull_requests:
+        # Ignore all of the _url fields
+        pull_request = {
+            key: value for key, value in original.items() if not key.endswith("url")
+        }
+        # Add repo key
+        pull_request["repo"] = repo["id"]
+        # Pull request _links can be flattened to just their URL
+        pull_request["url"] = pull_request["_links"]["html"]["href"]
+        pull_request.pop("_links")
+        # Extract user
+        pull_request["user"] = save_user(db, pull_request["user"])
+        labels = pull_request.pop("labels")
+        # Extract merged_by, if it exists
+        if pull_request.get("merged_by"):
+            pull_request["merged_by"] = save_user(db, pull_request["merged_by"])
+        # Head sha
+        pull_request["head"] = pull_request["head"]["sha"]
+        pull_request["base"] = pull_request["base"]["sha"]
+        # Extract milestone
+        if pull_request["milestone"]:
+            pull_request["milestone"] = save_milestone(
+                db, pull_request["milestone"], repo["id"]
+            )
+        # For the moment we ignore the assignees=[] array but we DO turn assignee
+        # singular into a foreign key reference
+        pull_request.pop("assignees", None)
+        if original["assignee"]:
+            pull_request["assignee"] = save_user(db, pull_request["assignee"])
+        pull_request.pop("active_lock_reason")
+        # ignore requested_reviewers and requested_teams
+        pull_request.pop("requested_reviewers", None)
+        pull_request.pop("requested_teams", None)
+        # Insert record
+        table = db["pull_requests"].insert(
+            pull_request,
+            pk="id",
+            foreign_keys=[
+                ("user", "users", "id"),
+                ("merged_by", "users", "id"),
+                ("assignee", "users", "id"),
+                ("milestone", "milestones", "id"),
+                ("repo", "repos", "id"),
+            ],
+            alter=True,
+            replace=True,
+            columns={
+                "user": int,
+                "assignee": int,
+                "milestone": int,
+                "repo": int,
+                "title": str,
+                "body": str,
+                "merged_by": int,
+            },
+        )
+        # m2m for labels
+        for label in labels:
+            table.m2m("labels", label, pk="id")
+
+
 def save_user(db, user):
     # Remove all url fields except avatar_url and html_url
     to_save = {
@@ -263,17 +336,38 @@ def save_license(db, license):
     return db["licenses"].insert(license, pk="key", replace=True).last_pk
 
 
-def fetch_issues(repo, token=None, issue=None):
+def fetch_issues(repo, token=None, issue_ids=None):
     headers = make_headers(token)
-    if issue is not None:
-        url = "https://api.github.com/repos/{}/issues/{}".format(repo, issue)
-        yield from [requests.get(url).json()]
+    headers["accept"] = "application/vnd.github.v3+json"
+    if issue_ids:
+        for issue_id in issue_ids:
+            url = "https://api.github.com/repos/{}/issues/{}".format(repo, issue_id)
+            response = requests.get(url, headers=headers)
+            response.raise_for_status()
+            yield response.json()
     else:
         url = "https://api.github.com/repos/{}/issues?state=all&filter=all".format(repo)
         for issues in paginate(url, headers):
             yield from issues
 
 
+def fetch_pull_requests(repo, token=None, pull_request_ids=None):
+    headers = make_headers(token)
+    headers["accept"] = "application/vnd.github.v3+json"
+    if pull_request_ids:
+        for pull_request_id in pull_request_ids:
+            url = "https://api.github.com/repos/{}/pulls/{}".format(
+                repo, pull_request_id
+            )
+            response = requests.get(url, headers=headers)
+            response.raise_for_status()
+            yield response.json()
+    else:
+        url = "https://api.github.com/repos/{}/pulls?state=all&filter=all".format(repo)
+        for pull_requests in paginate(url, headers):
+            yield from pull_requests
+
+
 def fetch_issue_comments(repo, token=None, issue=None):
     assert "/" in repo
     headers = make_headers(token)
@@ -674,3 +768,91 @@ def fetch_readme(token, full_name, html=False):
         return response.text
     else:
         return base64.b64decode(response.json()["content"]).decode("utf-8")
+
+
+def fetch_workflows(token, full_name):
+    headers = make_headers(token)
+    url = "https://api.github.com/repos/{}/contents/.github/workflows".format(full_name)
+    response = requests.get(url, headers=headers)
+    if response.status_code == 404:
+        return {}
+    workflows = {}
+    for item in response.json():
+        name = item["name"]
+        content = requests.get(item["download_url"]).text
+        workflows[name] = content
+    return workflows
+
+
+def save_workflow(db, repo_id, filename, content):
+    workflow = yaml.safe_load(content)
+    jobs = workflow.pop("jobs", None) or {}
+    # If there's a `True` key it was probably meant to be "on" - grr YAML
+    if True in workflow:
+        workflow["on"] = workflow.pop(True)
+    # Replace workflow if one exists already
+    existing = list(
+        db["workflows"].rows_where("repo = ? and filename = ?", [repo_id, filename])
+    )
+    if existing:
+        # Delete jobs, steps and this record
+        existing_id = existing[0]["id"]
+        db["steps"].delete_where(
+            "job in (select id from jobs where workflow = ?)", [existing_id]
+        )
+        db["jobs"].delete_where("workflow = ?", [existing_id])
+        db["workflows"].delete_where("id = ?", [existing_id])
+    workflow_id = (
+        db["workflows"]
+        .insert(
+            {
+                **workflow,
+                **{
+                    "repo": repo_id,
+                    "filename": filename,
+                    "name": workflow.get("name", filename),
+                },
+            },
+            pk="id",
+            column_order=["id", "filename", "name"],
+            alter=True,
+            foreign_keys=["repo"],
+        )
+        .last_pk
+    )
+    db["workflows"].create_index(["repo", "filename"], unique=True, if_not_exists=True)
+    for job_name, job_details in jobs.items():
+        steps = job_details.pop("steps", None) or []
+        job_id = (
+            db["jobs"]
+            .insert(
+                {
+                    **{
+                        "workflow": workflow_id,
+                        "name": job_name,
+                        "repo": repo_id,
+                    },
+                    **job_details,
+                },
+                pk="id",
+                alter=True,
+                foreign_keys=["workflow", "repo"],
+            )
+            .last_pk
+        )
+        db["steps"].insert_all(
+            [
+                {
+                    **{
+                        "seq": i + 1,
+                        "job": job_id,
+                        "repo": repo_id,
+                    },
+                    **step,
+                }
+                for i, step in enumerate(steps)
+            ],
+            alter=True,
+            pk="id",
+            foreign_keys=["job", "repo"],
+        )
